@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2023 The NATS Authors
+ * Copyright 2018-2024 The NATS Authors
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -17,6 +17,7 @@ package jwt
 
 import (
 	"fmt"
+	"strings"
 	"testing"
 	"time"
 
@@ -708,6 +709,40 @@ func TestAccountMapping(t *testing.T) { // don't block encoding!!!
 	}
 }
 
+func TestAccountClusterMany100MappingOK(t *testing.T) { // don't block encoding!!!
+	akp := createAccountNKey(t)
+	apk := publicKey(akp, t)
+
+	account := NewAccountClaims(apk)
+	vr := &ValidationResults{}
+
+	account.AddMapping("q",
+		WeightedMapping{Subject: "qq", Weight: 100, Cluster: "A"},
+		WeightedMapping{Subject: "qq", Weight: 100, Cluster: "B"},
+		WeightedMapping{Subject: "bb", Weight: 100, Cluster: "C"},
+		WeightedMapping{Subject: "qq", Weight: 100})
+	account.Validate(vr)
+	if !vr.IsEmpty() {
+		t.Fatal("Expected no errors")
+	}
+}
+
+func TestAccountClusterNoOver100Mapping(t *testing.T) { // don't block encoding!!!
+	akp := createAccountNKey(t)
+	apk := publicKey(akp, t)
+
+	account := NewAccountClaims(apk)
+	vr := &ValidationResults{}
+
+	account.AddMapping("q",
+		WeightedMapping{Subject: "qq", Weight: 100, Cluster: "A"},
+		WeightedMapping{Subject: "qq", Weight: 5, Cluster: "A"})
+	account.Validate(vr)
+	if !vr.IsBlocking(false) {
+		t.Fatal("Expected blocking error as sum of weights is > 100")
+	}
+}
+
 func TestAccountExternalAuthorization(t *testing.T) {
 	akp := createAccountNKey(t)
 	apk := publicKey(akp, t)
@@ -876,3 +911,150 @@ func TestAccountClaims_GetTags(t *testing.T) {
 		t.Fatal("expected tag bar")
 	}
 }
+
+func TestAccountClaimsTraceDest(t *testing.T) {
+	akp := createAccountNKey(t)
+	apk := publicKey(akp, t)
+
+	account := NewAccountClaims(apk)
+	for i, test := range []struct {
+		name        string
+		invalidSubj Subject
+		expectErr   bool
+	}{
+		{"trace not specified", "", false},
+		{"trace created but with empty destination", "", true},
+		{"trace dest has spaces", "invalid dest", true},
+		{"trace dest start with a dot", ".invalid.dest", true},
+		{"trace dest ends with a dot", "invalid.dest.", true},
+		{"trace dest has consecutive dots", "invalid..dest", true},
+		{"trace dest invalid publish dest", "invalid.publish.*.dest", true},
+	} {
+		t.Run(test.name, func(t *testing.T) {
+			if i > 0 {
+				account.Trace = &MsgTrace{Destination: test.invalidSubj}
+			}
+			vr := CreateValidationResults()
+			account.Validate(vr)
+
+			if test.expectErr && vr.IsEmpty() {
+				t.Fatal("account validation should have failed")
+			} else if !test.expectErr && !vr.IsEmpty() {
+				t.Fatalf("account validation should not have failed, got %+v", vr.Issues)
+			}
+		})
+	}
+}
+
+func TestAccountClaimsTraceDestSampling(t *testing.T) {
+	akp := createAccountNKey(t)
+	apk := publicKey(akp, t)
+
+	account := NewAccountClaims(apk)
+	for _, test := range []struct {
+		name      string
+		dest      string
+		sampling  int
+		expectErr string
+	}{
+		{"sampling without destination", "", 10, "subject cannot be empty"},
+		{"sampling negative", "dest", -1, "should be in the range [1..100]"},
+		{"sampling above 100", "dest", 101, "should be in the range [1..100]"},
+		{"sampling at 50", "dest", 50, ""},
+		{"sampling at zero sets to 100", "dest", 0, ""},
+	} {
+		t.Run(test.name, func(t *testing.T) {
+			account.Trace = &MsgTrace{Destination: Subject(test.dest), Sampling: test.sampling}
+			vr := CreateValidationResults()
+			account.Validate(vr)
+
+			if test.expectErr == "" {
+				if !vr.IsEmpty() {
+					t.Fatalf("account validation should not have failed, got %+v", vr.Issues)
+				}
+				if test.sampling == 0 {
+					if account.Trace.Sampling != 100 {
+						t.Fatalf("account sampling should have been set to 100 got %d", account.Trace.Sampling)
+					}
+				} else if test.sampling != account.Trace.Sampling {
+					t.Fatalf("account sampling should be %d, got %d", test.sampling, account.Trace.Sampling)
+				}
+			} else {
+				if vr.IsEmpty() {
+					t.Fatal("account validation should have failed")
+				}
+				if !strings.Contains(vr.Issues[0].Description, test.expectErr) {
+					t.Fatalf("account validation should have failed with error %q, got %q", test.expectErr, vr.Issues[0].Description)
+				}
+			}
+		})
+	}
+}
+
+func TestClusterTraffic_Valid(t *testing.T) {
+	type clustertest struct {
+		input string
+		ok    bool
+	}
+
+	tests := []clustertest{
+		{input: "", ok: true},
+		{input: "system", ok: true},
+		{input: "SYSTEM", ok: false},
+		{input: "owner", ok: true},
+		{input: "OWNER", ok: false},
+		{input: "unknown", ok: false},
+		{input: "account", ok: false},
+	}
+
+	for _, test := range tests {
+		ct := ClusterTraffic(test.input)
+		err := ct.Valid()
+		if test.ok && err != nil {
+			t.Fatalf("unexpected err for input %q: %v", test.input, err)
+		}
+		if !test.ok && err == nil {
+			t.Fatalf("expected to fail input %q", test.input)
+		}
+	}
+}
+
+func TestSignFn(t *testing.T) {
+	okp := createOperatorNKey(t)
+	opub := publicKey(okp, t)
+	opk, err := nkeys.FromPublicKey(opub)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	akp := createAccountNKey(t)
+	pub := publicKey(akp, t)
+
+	var ok bool
+	ac := NewAccountClaims(pub)
+	ac.Name = "A"
+	s, err := ac.EncodeWithSigner(opk, func(pub string, data []byte) ([]byte, error) {
+		if pub != opub {
+			t.Fatal("expected pub key in callback to match")
+		}
+		ok = true
+		return okp.Sign(data)
+	})
+
+	if err != nil {
+		t.Fatal("error encoding")
+	}
+	if !ok {
+		t.Fatal("expected ok to be true")
+	}
+
+	ac, err = DecodeAccountClaims(s)
+	if err != nil {
+		t.Fatal("error decoding encoded jwt")
+	}
+	vr := CreateValidationResults()
+	ac.Validate(vr)
+	if !vr.IsEmpty() {
+		t.Fatalf("claims validation should not have failed, got %+v", vr.Issues)
+	}
+}
